@@ -63,7 +63,9 @@ MAT_SHIN2 = 0xA041  # Shininess of the object/material (percent)
 MAT_SHIN3 = 0xA042  # Reflection of the object/material (percent)
 MAT_TRANSPARENCY = 0xA050  # Transparency value of material (percent)
 MAT_SELF_ILLUM = 0xA080  # Self Illumination value of material
+MAT_SELF_ILPCT = 0xA084  # Self illumination strength (percent)
 MAT_WIRE = 0xA085  # Only render's wireframe
+MAT_SHADING = 0xA100  # Material shading method
 
 MAT_TEXTURE_MAP = 0xA200  # This is a header for a new texture map
 MAT_SPECULAR_MAP = 0xA204  # This is a header for a new specular map
@@ -300,7 +302,21 @@ def add_texture_to_material(image, contextWrapper, pct, extend, alpha, scale, of
     elif extend == 'noWrap':
         img_wrap.extension = 'CLIP'
     if alpha == 'alpha':
-        links.new(img_wrap.node_image.outputs['Alpha'], img_wrap.socket_dst)
+        for link in links:
+            if link.from_node.type == 'TEX_IMAGE' and link.to_node.type == 'MIX_RGB':
+                tex = link.from_node.image.name
+                own_node = img_wrap.node_image
+                own_map = img_wrap.node_mapping
+                if tex == image.name:
+                    links.new(link.from_node.outputs['Alpha'], img_wrap.socket_dst)
+                    nodes.remove(own_map)
+                    nodes.remove(own_node)
+                    for imgs in bpy.data.images:
+                        if imgs.name[-3:].isdigit():
+                            if not imgs.users:
+                                bpy.data.images.remove(imgs)
+                else:
+                    links.new(img_wrap.node_image.outputs['Alpha'], img_wrap.socket_dst)
 
     shader.location = (300, 300)
     contextWrapper._grid_to_location(1, 0, dst_node=contextWrapper.node_out, ref_node=shader)
@@ -321,7 +337,7 @@ def process_next_chunk(context, file, previous_chunk, imported_objects, IMAGE_SE
     contextMesh_smooth = None
     contextMeshUV = None
 
-    TEXTURE_DICT = {}
+    #TEXTURE_DICT = {}
     MATDICT = {}
 
     # Localspace variable names, faster.
@@ -377,23 +393,19 @@ def process_next_chunk(context, file, previous_chunk, imported_objects, IMAGE_SE
                 else:
                     bmat = MATDICT.get(matName)
                     # in rare cases no materials defined.
-                    if bmat:
-                        img = TEXTURE_DICT.get(bmat.name)
-                    else:
-                        print("    warning: material %r not defined!" % matName)
-                        bmat = MATDICT[matName] = bpy.data.materials.new(matName)
-                        img = None
 
                 bmesh.materials.append(bmat)  # can be None
+                for fidx in faces:
+                    bmesh.polygons[fidx].material_index = mat_idx
 
-                if uv_faces and img:
-                    for fidx in faces:
-                        bmesh.polygons[fidx].material_index = mat_idx
-                        # TODO: How to restore this?
-                        # uv_faces[fidx].image = img
-                else:
-                    for fidx in faces:
-                        bmesh.polygons[fidx].material_index = mat_idx
+#                if uv_faces and img:
+#                    for fidx in faces:
+#                        bmesh.polygons[fidx].material_index = mat_idx
+#                        # TODO: How to restore this?
+#                        # uv_faces[fidx].image = img
+#                else:
+#                    for fidx in faces:
+#                        bmesh.polygons[fidx].material_index = mat_idx
 
             if uv_faces:
                 uvl = bmesh.uv_layers.active.data[:]
@@ -465,6 +477,7 @@ def process_next_chunk(context, file, previous_chunk, imported_objects, IMAGE_SE
         pct = 50
 
         contextWrapper.emission_color = contextMaterial.line_color[:3]
+        contextWrapper.emission_strength = contextMaterial.line_priority / 100
         contextWrapper.base_color = contextMaterial.diffuse_color[:3]
         contextWrapper.specular = contextMaterial.specular_intensity
         contextWrapper.roughness = contextMaterial.roughness
@@ -478,7 +491,7 @@ def process_next_chunk(context, file, previous_chunk, imported_objects, IMAGE_SE
 
             elif temp_chunk.ID == MAT_MAP_FILEPATH:
                 texture_name, read_str_len = read_string(file)
-                img = TEXTURE_DICT[contextMaterial.name] = load_image(texture_name, dirname, recursive=IMAGE_SEARCH)
+                img = load_image(texture_name, dirname, place_holder=False, recursive=IMAGE_SEARCH, check_existing=True)
                 temp_chunk.bytes_read += read_str_len  # plus one for the null character that gets removed
 
             elif temp_chunk.ID == MAT_MAP_USCALE:
@@ -667,6 +680,33 @@ def process_next_chunk(context, file, previous_chunk, imported_objects, IMAGE_SE
                 print("Cannot read material transparency")
             new_chunk.bytes_read += temp_chunk.bytes_read
 
+        elif new_chunk.ID == MAT_SELF_ILPCT:
+            read_chunk(file, temp_chunk)
+            if temp_chunk.ID == PERCENTAGE_SHORT:
+                temp_data = file.read(SZ_U_SHORT)
+                temp_chunk.bytes_read += SZ_U_SHORT
+                contextMaterial.line_priority = int(struct.unpack('H', temp_data)[0])
+            elif temp_chunk.ID == PERCENTAGE_FLOAT:
+                temp_data = file.read(SZ_FLOAT)
+                temp_chunk.bytes_read += SZ_FLOAT
+                contextMaterial.line_priority = (float(struct.unpack('f', temp_data)[0]) * 100)
+            new_chunk.bytes_read += temp_chunk.bytes_read
+
+        elif new_chunk.ID == MAT_SHADING:
+            shading = read_short(new_chunk)
+            if shading >= 2:
+                contextWrapper.use_nodes = True
+                contextWrapper.emission_color = contextMaterial.line_color[:3]
+                contextWrapper.emission_strength = contextMaterial.line_priority / 100
+                contextWrapper.base_color = contextMaterial.diffuse_color[:3]
+                contextWrapper.specular = contextMaterial.specular_intensity
+                contextWrapper.roughness = contextMaterial.roughness
+                contextWrapper.metallic = contextMaterial.metallic
+                contextWrapper.alpha = contextMaterial.diffuse_color[3]
+                contextWrapper.use_nodes = False
+                if shading >= 3:
+                    contextWrapper.use_nodes = True
+
         elif new_chunk.ID == MAT_TEXTURE_MAP:
             read_texture(new_chunk, temp_chunk, "Diffuse", "COLOR")
 
@@ -684,9 +724,15 @@ def process_next_chunk(context, file, previous_chunk, imported_objects, IMAGE_SE
             read_texture(new_chunk, temp_chunk, "Bump", "NORMAL")
 
         elif new_chunk.ID == MAT_BUMP_PERCENT:
-            temp_data = file.read(SZ_U_SHORT)
-            new_chunk.bytes_read += SZ_U_SHORT
-            contextWrapper.normalmap_strength = (float(struct.unpack('<H', temp_data)[0]) / 100)
+            read_chunk(file, temp_chunk)
+            if temp_chunk.ID == PERCENTAGE_SHORT:
+                temp_data = file.read(SZ_U_SHORT)
+                temp_chunk.bytes_read += SZ_U_SHORT
+                contextWrapper.normalmap_strength = (float(struct.unpack('<H', temp_data)[0]) / 100)
+            elif temp_chunk.ID == PERCENTAGE_FLOAT:
+                temp_data = file.read(SZ_FLOAT)
+                temp_chunk.bytes_read += SZ_FLOAT
+                contextWrapper.normalmap_strength = float(struct.unpack('f', temp_data)[0])
             new_chunk.bytes_read += temp_chunk.bytes_read
 
         elif new_chunk.ID == MAT_SHIN_MAP:
@@ -766,7 +812,7 @@ def process_next_chunk(context, file, previous_chunk, imported_objects, IMAGE_SE
             contextCamera.rotation_euler[2] = -1 * (math.radians(90) - math.acos(cam[0] / focus))
             new_chunk.bytes_read += SZ_FLOAT
             temp_data = file.read(SZ_FLOAT)
-            contextCamera.data.lens = (float(struct.unpack('f', temp_data)[0]) * 10)
+            contextCamera.data.lens = float(struct.unpack('f', temp_data)[0])
             new_chunk.bytes_read += SZ_FLOAT
             contextMatrix = None  # Reset matrix
             CreateBlenderObject = False
